<?php 
/**
* @package TinyFileHost
* @copyright Copyright (C) 2008-2011 Clodo. All rights reserved.
* @license http://www.gnu.org/copyleft/lgpl.html GNU/LGPL, see LICENSE.php
* This program is free software; you can redistribute it and/or modify it under 
* the terms of the GNU Lesser General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.

* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

* You should have received a copy of the GNU Lesser General Public License along with
* this program; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place - Suite 330, Boston, MA 02111-1307, USA, or go to
* http://www.gnu.org/copyleft/lesser.txt.
*/

include("conf.php");

/* --------------------------------------------
General functions
-------------------------------------------- */

function error($msg)
{
	throw new Exception($msg);
}

function echocr($str)
{
	echo $str . "\r\n";
}

function escapeSql($text)
{
	return mysql_real_escape_string($text);
}

/* --------------------------------------------
Functions
-------------------------------------------- */

$pageHeaderGenerated = false;

function generateHeader()
{
	global $CONFIG;
	global $pageHeaderGenerated;
	
	if($pageHeaderGenerated == false)
	{
		header("Content-type: text/html; charset=UTF-8");
		echocr("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
		echocr("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
		echocr("<head>");	
		echocr("<title>" . $CONFIG["title"] . "</title>");
		echocr("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />");
		if(isset($CONFIG["external_css"]))
			echocr("<link href=\"" . $CONFIG["external_css"] . "\" rel=\"stylesheet\" type=\"text/css\" />");	
		echocr("<link href=\"stylesheet.css\" rel=\"stylesheet\" type=\"text/css\" />");	
		echocr("<link href=\"skin.css\" rel=\"stylesheet\" type=\"text/css\" />");	
		echocr("</head>");
		echocr("<body>");	
		
		if(isset($CONFIG["logo"]))
		{
			echocr("<br /><a href=\"" . $CONFIG["url_home"] . "\"><img src=\"" . $CONFIG["logo"] . "\" alt=\"" . $CONFIG["title"] . "\" /></a>");
			echocr("<br />");
			echocr("<br />");
		}
		
		$pageHeaderGenerated = true;
	}
}

function generateFooter()
{
	global $CONFIG;
	global $pageHeaderGenerated;
	
	if($pageHeaderGenerated == true)
	{
		echocr("<div class=\"footer\">Tiny File Hoster v" . $CONFIG["version"] . " - <a href=\"?act=readme\">more...</a></div>");
		
		echocr("</body>");
		echocr("</html>");	
	}
}

function extensionFromType($imageType)
{
	switch($imageType)
	{
		case 1: return "gif";
		case 2: return "jpg";
		case 3: return "png";
	}
	
	return "unknown";
}

function extensionFromID($id, $connection)
{
	global $CONFIG;
	
	// Note: This isn't the best solution, but this function is called only if DB isn't used.
	if($connection)
	{
		$sql = "select image_format from tfh_images where id='" . escapeSql($id) . "'";					
		$result = mysql_query($sql, $connection);		
		$row = mysql_fetch_array($result) or error(mysql_error());	
		return extensionFromType($row["image_format"]);
	}
	else
	{
		if(file_exists($CONFIG["path_images"] . $id . ".gif"))
			return "gif";
		if(file_exists($CONFIG["path_images"] . $id . ".jpg"))
			return "jpg";
		if(file_exists($CONFIG["path_images"] . $id . ".png"))
			return "png";
	}
	
	error("Unknown error.");
}

function processFile($path, $connection)
{
	global $CONFIG;
	
	$bytes_orig = filesize($path);
	$id = sha1_file($path);
	$width_orig = 0;
	$height_orig = 0;
	$image_type = -1;
	$imagePath = $CONFIG["path_images"] . $id;
	$thumbPath = $CONFIG["path_thumbs"] . $id;					
	
	if ($bytes_orig>$CONFIG["max_size"]) 
		throw new Exception("File too big.");		
	
	// Check if is an image.
	// If other types are enabled, you need to add the code checking.
	// This because we can't trust ["type"] , it's given by client.
	
	try
	{								
		list($width_orig, $height_orig, $image_type) = getimagesize($path);
		
		if( ($image_type < 1) || ($image_type > 3) )
			throw new Exception("Unsupported filetype!");
			
		if($width_orig <= 0)
			throw new Exception("Unsupported filetype!");
			
		if($height_orig <= 0)
			throw new Exception("Unsupported filetype!");
		
		switch ($image_type) 
		{
			case 1: $img = @imagecreatefromgif($path); break;
			case 2: $img = @imagecreatefromjpeg($path);  break;
			case 3: $img = @imagecreatefrompng($path); break;						
		}
		
		$ext = extensionFromType($image_type);
		
		$thumbPath .= "." . $ext;
		$imagePath .= "." . $ext;
		
		if($img == null)
			throw new Exception("Unsupported filetype!");
				
		if($path != $imagePath)
			copy($path, $imagePath);
		
		
		// Thumbnail
		if(file_exists($thumbPath))
				unlink($thumbPath);
				
		$thumbWidth = $CONFIG["thumb_width"];
		$thumbHeight = $thumbWidth; // Only square thumbnail supported.
		
		if( ($width_orig > $thumbWidth) || ($height_orig > $thumbHeight) )
		{
			$sx = $width_orig / $thumbWidth;
			$sy = $height_orig / $thumbHeight;

			if($sx>$sy)
			{
				$thumb_width = $thumbWidth;
				$thumb_height = ($height_orig*$thumbWidth)/$width_orig;
			}
			else
			{
				$thumb_height = $thumbHeight;
				$thumb_width = ($width_orig*$thumbHeight)/$height_orig;
			}	
			
			$thumb_width = intval($thumb_width);
			$thumb_height = intval($thumb_height);
			
			if($thumb_width == 0)
				$thumb_width++;
			
			if($thumb_height == 0)
				$thumb_height++;			
		
			$newImg = imagecreatetruecolor($thumb_width, $thumb_height);
				
			// Check if this image is PNG or GIF, then set if Transparent
			if(($image_type == 1) OR ($image_type==3))
			{
				imagealphablending($newImg, false);
				imagesavealpha($newImg,true);
				$transparent = imagecolorallocatealpha($newImg, 255, 255, 255, 127);
				imagefilledrectangle($newImg, 0, 0, $thumb_width, $thumb_height, $transparent);
			}
			imagecopyresampled($newImg, $img, 0, 0, 0, 0, $thumb_width, $thumb_height, $width_orig, $height_orig);
			
			$thumbText = $width_orig . " x " . $height_orig . " | " . extensionFromType($image_type);
			
			$font = str_replace("index.php","visitor1.ttf",$_SERVER['SCRIPT_FILENAME']);
			$fontSize = 7.58;
			$black = imagecolorallocate($newImg, 1, 1, 1);
			$white = imagecolorallocate($newImg, 255, 255, 255);

			$tb = imagettfbbox($fontSize, 0, $font, $thumbText);
			
			$x = ceil($thumb_width-$tb[2])-3;
			$y = ceil($thumb_height-$tb[3])-3;
			
			for($ox=-1;$ox<=1;$ox++)
			{
				for($oy=-1;$oy<=1;$oy++)
				{
					imagettftext($newImg, $fontSize, 0, $x+$ox, $y+$oy, -$black, $font, $thumbText);
				}
			}
			imagettftext($newImg, $fontSize, 0, $x, $y, -$white, $font, $thumbText);
						
			// Generate the file, and rename it to $newfilename
			switch ($image_type) 
			{
				case 1: imagegif($newImg,$thumbPath); break;
				case 2: imagejpeg($newImg,$thumbPath);  break;
				case 3: imagepng($newImg,$thumbPath); break;
					throw new Exception("Failed resize image.");					
			}						
		}
	}
	catch(Exception $e)
	{
		if(file_exists($imagePath))
			unlink($imagePath);
		if(file_exists($thumbPath))
			unlink($thumbPath);		
		throw new Exception($e->getMessage());			
	}
				
	// Database update
	if($connection)
	{
		$sql = "insert ignore into tfh_images (id, author_ip, author_ua, creation_date, image_format, image_width, image_height, bytes) values (";
		$sql .= "'" . escapeSql($id) . "',";
		$sql .= "'" . escapeSql($_SERVER['REMOTE_ADDR']) . "',";
		$sql .= "'" . escapeSql($_SERVER['HTTP_USER_AGENT']) . "',";
		$sql .= "now(),";
		$sql .= escapeSql($image_type) . ",";
		$sql .= escapeSql($width_orig) . ",";
		$sql .= escapeSql($height_orig) . ",";
		$sql .= escapeSql($bytes_orig) . ")";
		
		$result = mysql_query($sql,$connection);		
	}
	
	return $id;
}

/* --------------------------------------------
Main
-------------------------------------------- */

function main()
{
	global $CONFIG;

	$CONFIG["version"] = "2.0";
	
	$connection = null;
	
	try
	{
		if(isset($_GET["id"]))
		{
			$id = strtolower($_GET["id"]);
			
			if( (strlen($id) == 10) || (strlen($id) == 14) ) // Compatibility 0.X
			{
				if(strlen($id) == 10)
					$id .= "." . $_GET["ext"];
				$imagePath = $CONFIG["path_images"] . $id;
				if(file_exists($imagePath))
					$id = sha1_file($imagePath);
			}
			
			$id = str_replace(".png","",$id); // Compatibility 1.X
			$id = str_replace(".jpg","",$id); // Compatibility 1.X
			$id = str_replace(".gif","",$id); // Compatibility 1.X
			
			if(preg_match('/^[a-z0-9]*.[a-z0-9]*$/i', $id) == false)
					throw new Exception("Invalid id.");			
		}
		else
			$id = "";
			
		if(isset($_SERVER['HTTP_REFERER']))
			$referer = $_SERVER['HTTP_REFERER'];
		else
			$referer = "";
			
		if(isset($_GET["pwd"]))
			$adminPwd = $_GET["pwd"];
		else
			$adminPwd = "";
			
		$action = "";
		if (isset($_FILES['myfile'])) 
			$action = "upload";
		else if (isset($__POST['myurl'])) 
			$action = "upload";	
		else if (isset($_GET["act"]))
		{
			$action = $_GET["act"];
			
			if( ($CONFIG["list_allowed"] == false) && ($action == "lists") && ($adminPwd != $CONFIG["admin_password"]) )
				$action = "";
				
			if( ($CONFIG["list_allowed"] == false) && ($action == "thumbs") && ($adminPwd != $CONFIG["admin_password"]) )
				$action = "";
		}
		else if ($id != "")
			$action = "view";
			
		if($action == "")
			$action = "wizard";
			
		// Open Database, if possible	
		if(isset($CONFIG["db_host"]))
		{
			$connection = @mysql_connect($CONFIG["db_host"], $CONFIG["db_user"], $CONFIG["db_password"]) or error("Unable to connect to database: ".mysql_error());				
			$database = @mysql_select_db($CONFIG["db_name"], $connection) or error("Unable to select the database: ".mysql_error());
		}
			
		//Check if GD extension is loaded
		if (!extension_loaded('gd') && !extension_loaded('gd2')) 
		{
			throw new Exception("GD is not loaded.");		
		}
		
		if ($action == "upload")
		{
			if(isset($CONFIG["password"]) && ($CONFIG["password"] != ""))
			{
				if($CONFIG["password"] != $_POST["password"])
					throw new Exception("Invalid password.");
			}
			
			if( (!isset($_FILES['myfile'])) && (!isset($_POST['myurl'])) )
				throw new Exception("File not uploaded or missing url.");
				
			$tempFile = $CONFIG["path_temp"] . time() . ".tmp";
											
			if (is_uploaded_file($_FILES['myfile']['tmp_name']) == true)
			{	
				$tempPath = $_FILES["myfile"]["tmp_name"];
				$tempName = $_FILES["myfile"]["name"];
				
				$ext = substr($tempName, strrpos($tempName, ".") +1);
				if(preg_match('/^[a-z0-9]*$/i', $ext) == false)
					throw new Exception("Invalid extension.");
					
				$res = move_uploaded_file($tempPath, $tempFile);
	
				if (!$res) 
					throw new Exception("Unknown error.");		
			}
			else
			{
				$remoteUrl = trim($_POST["myurl"]);
				
				/*
				$ext = substr($remoteUrl, strrpos($remoteUrl, ".") +1);
				if(preg_match('/^[a-z0-9]*$/i', $ext) == false)
					throw new Exception("Invalid extension.");
				*/
				
				$data = file_get_contents($remoteUrl, 0, NULL, 0, $CONFIG["max_size"]);
				file_put_contents($tempFile, $data, 0);			
			}
			
			if(file_exists($tempFile)) 
			{				
				try
				{
					$id = processFile($tempFile, $connection);				
					
					$viewUrl = $CONFIG["url_base"] . "?id=" . $id;
								
					unlink($tempFile);
				
					header("Location: " . $viewUrl);
				}
				catch(Exception $e)
				{
					unlink($tempFile);
					
					throw new Exception($e->getMessage());			
				}
			}			
		}
		else if ($action == "image")
		{
			// Database update
			if($connection)
			{
				$sql = "update tfh_images set lastvisit_image_date=now(), lastvisit_image_referer='" . escapeSql($referer) . "', visit_image_counter=visit_image_counter+1 where id='" . escapeSql($id) . "'";
				$result = mysql_query($sql,$connection);					
			}
			
			$ext = extensionFromID($id, $connection);
			
			$path = $CONFIG["path_images"] . $id . "." . $ext;		
			$data = file_get_contents($path);
			
			header("Content-type: image/" . $ext);
			header("Last-Modified: " . gmdate("D, d M Y H:i:s", filemtime($path)) . " GMT");
			header("Content-Transfer-Encoding: binary");
			header("Content-Length: " . filesize($path) . ";\n");
			echo $data;
		}
		else if ($action == "thumb")
		{
			// Database update
			if($connection)
			{
				$sql = "update tfh_images set lastvisit_thumb_date=now(), lastvisit_thumb_referer='" . escapeSql($referer) . "', visit_thumb_counter=visit_thumb_counter+1 where id='" . escapeSql($id) . "'";
				$result = mysql_query($sql, $connection);					
			}
			
			$ext = extensionFromID($id, $connection);
			
			$path = $CONFIG["path_thumbs"] . $id . "." . $ext;		
			$data = file_get_contents($path);
			
			header("Content-type: image/" . $ext);
			header("Last-Modified: " . gmdate("D, d M Y H:i:s", filemtime($path)) . " GMT");
			header("Content-Transfer-Encoding: binary");
			header("Content-Length: " . filesize($path) . ";\n");
			echo $data;
		}
		else
		{
			generateHeader();
				
			if($action=="readme")
			{
				$text = file_get_contents("readme.txt");
				$text = str_replace("\n","<br/>", $text);
				echo "<div class=\"readme\">" . $text . "</div>";
			}
			else if($action=="lists")
			{
				$dir = opendir($CONFIG["path_images"]);
				echo "<div class=\"lists_full\">";
				while($id = readdir($dir)) 
				{			
					if( ($id != ".") && ($id != "..") && ($id != "index.html") && ($id != ".htaccess") )
					{
						$viewUrl = $CONFIG["url_base"] . "?id=" . $id;
						
						echo "<div class=\"lists_full_img\">";
						echo "<a target=\"_blank\" href=\"" . $viewUrl . "\">";
						echo "<img src=\"" . $CONFIG["url_base"] . $CONFIG["path_images"] . $id . "\"/>";
						echo "</a>";
						echo "</div>\n";
					}
				}
				echo "</div>";
			
				closedir($dir);
			}
			else if($action=="thumbs")
			{
				$dir = opendir($CONFIG["path_images"]);
				echo "<table class=\"lists_thumbs\">";
				$index = 0;
				while($id = readdir($dir)) 
				{			
					if( ($id != ".") && ($id != "..") && ($id != "index.html") && ($id != ".htaccess") )
					{
						$viewUrl = $CONFIG["url_base"] . "?id=" . $id;
						
						$thumbPath = $CONFIG["path_thumbs"] . $id;		
						
						$imgUrl = $CONFIG["url_base"] . $CONFIG["path_thumbs"] . $id;
						if(file_exists($thumbPath) == false)
							$imgUrl = $CONFIG["url_base"] . $CONFIG["path_images"] . $id;
							
						$index = $index % $CONFIG["thumbs_cols"];
						if($index == 0)
							echo "</tr>\n";
							
						$index++;
							
						if($index == 1)
							echo "<tr>";
						
						echo "<td class=\"lists_thumbs_img\">";
						echo "<a target=\"_blank\" href=\"" . $viewUrl . "\">";
						echo "<img src=\"" . $imgUrl . "\"/>";
						echo "</a>";
						echo "</td>";				
						
					}
				}
				
				if($index != 0)
					echo "</tr>\n";
				echo "</table>";
			
				closedir($dir);
			}
			else if($action=="wizard")
			{		
				echo "<div class=\"warning\">";
				echo $CONFIG["subtitle"];
				echo "</div>";
				echo "<div class=\"rules\">";
				echo "Select a file to upload. Max size: " . round($CONFIG["max_size"] / 1000) . " KB";
				echo "</div>";
				echo "<form method=\"post\" enctype=\"multipart/form-data\" action=\"index.php?act=upload\">";
				echo "<span>File: </span><input type=\"file\" name=\"myfile\" size=\"30\" /><br/><br/>";
				echo "<span>Url: </span><input type=\"text\" name=\"myurl\" size=\"50\" /><br/><br/>";
		
				if(isset($CONFIG["password"]) && ($CONFIG["password"] != ""))
					echo "<span>Password: </span><input type=\"text\" name=\"password\" size=\"10\" /> ";
		
				echo "<input type=\"submit\" value=\"Upload\" name=\"upload\"/>";
				echo "</form>";
			}
			else if ($action == "checking")
			{
				set_time_limit(0);
				
				if($adminPwd != $CONFIG["admin_password"])
					error("Administration only. Wrong or missing password.");
				
				$dir = opendir($CONFIG["path_images"]);
				echo "<div class=\"lists_full\">";
				
				$i = 0;
				while($id = readdir($dir)) 
				{			
					if( ($id != ".") && ($id != "..") && ($id != "index.html") && ($id != ".htaccess") )
					{
						$ext = substr($id, strrpos($id, ".") +1);
						$id = substr($id, 0, strrpos($id, "."));
						
						echo $id;
						echo " (" . $i . ")";
						echo " ... ";
						
						$path = $CONFIG["path_images"] . "/" . $id . "." . $ext;
												
						try
						{
							$id = processFile($path, $connection);				
							
							echo "Ok.";
						}
						catch(Exception $e)
						{
							unlink($path);
							
							echo "Error: " . $e->getMessage() . ". Deleted.";
						}
						
						echo "<br>";
						flush();
						
						$i++;
					}
				}
				echo "</div>";
			
				closedir($dir);
			}
			else if ($action == "view")
			{
				$ext = extensionFromID($id, $connection);
				
				$imagePath = $CONFIG["path_images"] . $id . "." . $ext;
				$thumbPath = $CONFIG["path_thumbs"] . $id . "." . $ext;
					
				if(file_exists($imagePath) == false)
					throw new Exception("File not found");
				
				$fullUrl = $CONFIG["url_base"] . $CONFIG["path_images"] . $id . "." . $ext;
				echo "<img src=\"" . $fullUrl . "\" alt=\"image\"/>";
				
				echo "<br>";				
				
				if($connection)
				{
					echo "<br>";
					$sql = "select UNIX_TIMESTAMP(creation_date) as creation_date, image_format, image_width, image_height, bytes, visit_image_counter from tfh_images where id='" . escapeSql($id) . "'";					
					$result = mysql_query($sql,$connection);		
					$row = mysql_fetch_array($result) or error(mysql_error());
					
					echo "Size: <b>" . $row["image_width"] . " x " . $row["image_height"] . "</b> | Type: <b>" . extensionFromType($row["image_format"]) . "</b> | Bytes: <b>" . $row["bytes"] . "</b> | Views: <b>" . $row["visit_image_counter"] . "</b> | Creation: <b>" . date('l jS \of F Y h:i:s A',$row['creation_date']) . "</b>";
					echo "<br>";
				}
				
				echo "<br>";
				
				$thumbUrl = "";
				if(file_exists($thumbPath))		
					$thumbUrl = $CONFIG["url_base"] . $CONFIG["path_thumbs"] . $id . "." . $ext;
				$viewUrl = $CONFIG["url_base"] . "?id=" . $id;
							
				echo "<script type=\"text/javascript\">\r\n";
				echo "<!--\r\n";
				echo "function highlight(field)\r\n";
				echo "{\r\n";
				echo "	field.focus();\r\n";
				echo "	field.select();\r\n";
				echo "}\r\n";
				echo "//-->\r\n";
				echo "</script>\r\n";
				$input = "<input style=\"width:100%\" type=\"text\" onclick=\"javascript:void(highlight(this));\" value=\"";
				echo "<table style=\"width:1000px;\">";						
				echo "<col style=\"width:200px;\">";		
				echo "<col style=\"width:800px;\">";		
				if($thumbUrl != "")
				{
					if($CONFIG["syntax_bbcode"])
						echo "<tr><td>BBCode Thumbnail:</td><td>" . $input . "[url=" . $viewUrl . "][img]" . $thumbUrl . "[/img][/url]" . "\" /></td></tr>";
					if($CONFIG["syntax_wiki"])
						echo "<tr><td>Wiki Thumbnail:</td><td>" . $input . "[[" . $viewUrl . " " . $thumbUrl . "]]" . "\" /></td></tr>";			
				}
				if($CONFIG["syntax_bbcode"])
					echo "<tr><td>BBCode Full:</td><td>" . $input . "[img]" . $fullUrl . "[/img]" . "\" /></td></tr>";		
				//if($CONFIG["syntax_wiki"]) // It's the same of Direct. Who know syntax wiki know that.
				//		echo "<tr><td>Wiki Full:</td><td>" . $input . "" . $fullUrl . "" . "\" /></td></tr>";		
				if($CONFIG["syntax_direct"])
					echo "<tr><td>Direct:</td><td>" . $input . $fullUrl . "\" /></td></tr>";
					
				echo "</table>";
				if($action != "view")
					echo "<a target=\"_blank\" href=\"" . $viewUrl . "\"><img src=\"" . $fullUrl . "\" alt=\"preview\"/></a>";
				if($thumbUrl != "")
				{
					echocr("<br/><br/><img src=\"" . $thumbUrl . "\" alt=\"image\"/>");
				}
			}			
		}
	}
	catch(Exception $e)
	{
		generateHeader();
		
		echo $e->getMessage();
	}
	
	generateFooter();
	
	if($connection)
		mysql_close($connection);	
	
}

// Calling main
main();

?>

